/**************************************************************************************
Author:         David Szybala
Website:        www.davidszybala.com
Alt Website:    www.twistedgerbil.com
Program:        geometry_properties.ms
Date:           10/11/10
Description:    This script allows the user to change the object properties on multiple
                objects at the same time.
***************************************************************************************/

try (destroyDialog geometry_properties_rollout) catch()


struct GeometryProperties
(
    display_names = #("See-Through", "Display as Box", "Backface Cull", "Edges Only",
                        "Vertex Ticks", "Trajectory", "Ignore Extents", "Show Frozen in Gray",
                        "Display by Layer", "Color by Layer"),
    display_properties = #("xray", "boxMode", "backFaceCull", "allEdges",
                            "vertexTicks", "showTrajectory", "ignoreExtents", "showFrozenInGray",
                            "displayByLayer", "colorByLayer"),
    render_names = #("Renderable", "Inherit Visibility", "Visible to Camera", "Visible to Reflect/Refract",
                        "Receive Shadows", "Cast Shadows", "Apply Atmospherics", "Render Occluded Objects",
                        "Render by Layer", "Motion Blur by Layer"),
    render_properties = #("renderable", "inheritVisibility", "primaryVisibility", "secondaryVisibility",
                            "receiveShadows", "castShadows", "applyAtmospherics", "renderOccluded",
                            "renderByLayer", "motionByLayer"),
    
    /**************************************************************************
    Sets the command panel to the create mode if the display mode is currently
    active. NOTE: Having the display panel open can severely slow down
    this scripts performance
    **************************************************************************/
    fn set_command_panel =
    (
        if (getCommandPanelTaskMode()) == #display do
            setCommandPanelTaskMode mode:#create
    ), --end fn
    
    /**************************************************************************
    Sets the display property
    **************************************************************************/
    fn set_display_property index state selection_only =
    (
        if selection_only then
            for current in selection do setProperty current display_properties[index] state
        else
            for current in geometry do setProperty current display_properties[index] state
    ), --end fn
    
    /**************************************************************************
    Sets the render property
    **************************************************************************/
    fn set_render_property index state selection_only =
    (
        if selection_only then
            for current in selection do setProperty current render_properties[index] state
        else
            for current in geometry do setProperty current render_properties[index] state
    ), --end fn
    
    /**************************************************************************
    Sets the render property
    **************************************************************************/
    fn set_object_id id_value selection_only =
    (
        if selection_only then
            for current in selection do current.gBufferChannel = id_value
        else
            for current in geometry do current.gBufferChannel = id_value
    ) --end fn
) --end struct


(--put script into local scope
    global geometry_properties_rollout
    local geometry_properties_ini_file = getDir #plugcfg + "\\GeometryProperties.ini"
    local geometry_properties_rollout_pos

    rollout geometry_properties_rollout "Geometry Properties"
    (
        local geo_props = GeometryProperties()
        
        groupbox display_group "Display" pos:[5,5] width:215 height:40
            dropDownList display_dropdown "" pos:[15,20] width:135 items:#("1")
            button display_on_button "ON" pos:[155,20] width:25 height:20
            button display_off_button "OFF" pos:[185,20] width:25 height:20
        
        groupbox render_group "Rendering" pos:[5,50] width:215 height:40
            dropDownList render_dropdown "" pos:[15,65] width:135 items:#("1")
            button render_on_button "ON" pos:[155,65] width:25 height:20
            button render_off_button "OFF" pos:[185,65] width:25 height:20
        
        groupbox id_group "G-Buffer" pos:[5,95] width:215 height:35
            label id_label "Object ID:" pos:[15,110] width:60 height:15
            spinner id_spinner "" pos:[75, 110] width:50 height:15 range:[0, 65535, 1] type:#integer scale:1
        
        groupbox option_group "Options" pos:[5,135] width:215 height:35
            checkbox selection_only_checkbox "Selection Only" pos:[15,150] width:100 height:15 checked:true
        
        label status_label "INFO: Standing by..." pos:[5,180] width:215 height:15
            
        /**********************************************************************
        UI open event handler. Prepares the UI for first time open
        **********************************************************************/
        on geometry_properties_rollout open do
        (
            display_dropdown.height = geo_props.display_names.count * 16
            display_dropdown.items = geo_props.display_names
            render_dropdown.height = geo_props.render_names.count * 16
            render_dropdown.items = geo_props.render_names
        ) --end on
        
        /**********************************************************************
        UI move event handler. Stores the curent position of the UI
        **********************************************************************/
        on geometry_properties_rollout moved pos do
            setIniSetting geometry_properties_ini_file "Dialog" "Position" (pos as string)
        
        /**********************************************************************
        Display on button event handler. Turns the display property on
        **********************************************************************/
        on display_on_button pressed do
        (
            geo_props.set_command_panel()
            geo_props.set_display_property display_dropdown.selection true selection_only_checkbox.checked
            status_label.text = "INFO: " + geo_props.display_names[display_dropdown.selection] + " is enabled"
        ) --end on
        
        /**********************************************************************
        Display off button event handler. Turns the display property off
        **********************************************************************/
        on display_off_button pressed do
        (
            geo_props.set_command_panel()
            geo_props.set_display_property display_dropdown.selection false selection_only_checkbox.checked
            status_label.text = "INFO: " + geo_props.display_names[display_dropdown.selection] + " is disabled"
        ) --end on
        
        /**********************************************************************
        Render on button event handler. Turns the render property on
        **********************************************************************/
        on render_on_button pressed do
        (
            geo_props.set_command_panel()
            geo_props.set_render_property render_dropdown.selection true selection_only_checkbox.checked
            status_label.text = "INFO: " + geo_props.render_names[render_dropdown.selection] + " is enabled"
        ) --end on
        
        /**********************************************************************
        Render off button event handler. Turns the render property off
        **********************************************************************/
        on render_off_button pressed do
        (
            geo_props.set_command_panel()
            geo_props.set_render_property render_dropdown.selection false selection_only_checkbox.checked
            status_label.text = "INFO: " + geo_props.render_names[render_dropdown.selection] + " is disabled"
        ) --end on
        
        /**********************************************************************
        ID spinner changed event handler. Assigns the defined object ID
        **********************************************************************/
        on id_spinner changed id_value do
        (
            geo_props.set_object_id id_value selection_only_checkbox.checked
            status_label.text = "INFO: Object ID set to " + (id_value as string)
        ) --end on
    )--end rollout
    
    geometry_properties_rollout_pos = execute (getIniSetting geometry_properties_ini_file "Dialog" "Position")
    if geometry_properties_rollout_pos == OK do geometry_properties_rollout_pos = [100,100]
    if geometry_properties_rollout_pos.x > sysinfo.desktopsize.x - 225 do
        geometry_properties_rollout_pos.x = sysinfo.desktopsize.x - 230
    if geometry_properties_rollout_pos.y > sysinfo.desktopsize.y - 255 do
        geometry_properties_rollout_pos.y = sysinfo.desktopsize.y - 255
    if geometry_properties_rollout_pos.x < 0 do geometry_properties_rollout_pos.x = 0 
    if geometry_properties_rollout_pos.y < 0 do geometry_properties_rollout_pos.y = 0
    createDialog geometry_properties_rollout 225 195 geometry_properties_rollout_pos.x geometry_properties_rollout_pos.y
)--end local scope
